@@ -1,16 +1,16 @@
-/*****************************************************************************
+/*============================================================================
                                   pamundice
-******************************************************************************
+==============================================================================
   Assemble a grid of images into one.
 
   By Bryan Henderson, San Jose CA 2001.01.31
 
   Contributed to the public domain.
-
-******************************************************************************/
+===========================================================================*/
 
 #include <assert.h>
 #include <string.h>
+#include <stdbool.h>
 
 #include "pm_c_util.h"
 #include "pam.h"
@@ -29,6 +29,8 @@ struct CmdlineInfo {
     unsigned int voverlap;
     const char * listfile;
     unsigned int listfileSpec;
+    const char * indexfile;
+    unsigned int indexfileSpec;
     unsigned int verbose;
 };
 
@@ -69,12 +71,14 @@ parseCommandLine(int argc, const char ** argv,
             &voverlapSpec,                    0);
     OPTENT3(0, "listfile",    OPT_STRING,  &cmdlineP->listfile,
             &cmdlineP->listfileSpec,          0);
+    OPTENT3(0, "indexfile",   OPT_STRING,  &cmdlineP->indexfile,
+            &cmdlineP->indexfileSpec,         0);
     OPTENT3(0, "verbose",     OPT_FLAG,    NULL,
             &cmdlineP->verbose,               0);
 
     opt.opt_table = option_def;
-    opt.short_allowed = FALSE;  /* We have no short (old-fashioned) options */
-    opt.allowNegNum = FALSE;  /* We have no parms that are negative numbers */
+    opt.short_allowed = false;  /* We have no short (old-fashioned) options */
+    opt.allowNegNum = false;  /* We have no parms that are negative numbers */
 
     pm_optParseOptions3( &argc, (char **)argv, opt, sizeof(opt), 0 );
         /* Uses and sets argc, argv, and some of *cmdline_p and others. */
@@ -97,6 +101,15 @@ parseCommandLine(int argc, const char ** argv,
     if (!voverlapSpec)
         cmdlineP->voverlap = 0;
 
+
+    if (cmdlineP->indexfileSpec) {
+        if (acrossSpec || downSpec)
+            pm_error ("You cannot specify -indexfile with -down or -across");
+        else if (cmdlineP->listfileSpec)
+            pm_error("You cannot specify both -listfile and -indexfile");
+
+    }
+
     if (cmdlineP->listfileSpec) {
         if (argc-1 > 0)
             pm_error("Program takes no parameters when -listfile is "
@@ -341,19 +354,19 @@ computeInputFileName(const char *  const pattern,
 
 static void
 createInFileListFmFile(const char  *  const listFile,
-                       unsigned int   const nRank,
-                       unsigned int   const nFile,
+                       unsigned int   const rankCt,
+                       unsigned int   const fileCt,
                        const char *** const inputFileListP) {
 
+    unsigned int const inFileCt = rankCt * fileCt;
     FILE * const lfP = pm_openr(listFile);
-    unsigned int const fileCt = nRank * nFile;
 
     const char ** inputFileList;
     unsigned int fileSeq;
 
-    MALLOCARRAY_NOFAIL(inputFileList, nRank * nFile);
+    MALLOCARRAY_NOFAIL(inputFileList, rankCt * fileCt);
 
-    for (fileSeq = 0; fileSeq < fileCt; ) {
+    for (fileSeq = 0; fileSeq < inFileCt; ) {
         int eof;
         size_t lineLen;
         char * buf = NULL;   /* initial value */
@@ -363,11 +376,9 @@ createInFileListFmFile(const char  *  const listFile,
 
         if (eof)
             pm_error("Premature EOF reading list file.  "
-                     "Read %u files.  Should be %u.", fileSeq, fileCt);
-        else if (lineLen > 0) {
-            inputFileList[fileSeq] = buf;
-            ++fileSeq;
-        }
+                     "Read %u files.  Should be %u.", fileSeq, inFileCt);
+        else if (lineLen > 0)
+            inputFileList[fileSeq++] = buf;
     }
     pm_close(lfP);
 
@@ -377,21 +388,88 @@ createInFileListFmFile(const char  *  const listFile,
 
 
 
+static void
+createInFileListFmIdxFile(const char *   const indexFileNm,
+                          const char   * const pattern,
+                          unsigned int * const rankCtP,
+                          unsigned int * const fileCtP,
+                          const char *** const inputFileListP) {
+
+    const char ** inputFileList;
+    unsigned int rankCt, fileCt;
+    unsigned int rank;
+    bool warnedSingleFile;
+
+    FILE * ifP;
+    struct pam indexPam;
+    tuple * indexRow;   /* Index row buffer */
+
+    ifP = pm_openr(indexFileNm);
+
+    pnm_readpaminit(ifP, &indexPam, PAM_STRUCT_SIZE(tuple_type));
+
+    rankCt = indexPam.height;
+    fileCt = indexPam.width;
+
+    indexRow = pnm_allocpamrow(&indexPam);
+
+    if (indexPam.depth < 2)
+        pm_error("Insufficient number of planes (%u) in index file.  "
+                 "Need %u.", indexPam.depth, 2);
+
+    MALLOCARRAY_NOFAIL(inputFileList, rankCt * fileCt);
+
+    for (rank = 0, warnedSingleFile = false; rank < rankCt; ++rank) {
+        unsigned int file;
+
+         pnm_readpamrow(&indexPam, indexRow);
+
+         for (file = 0; file < fileCt; ++file) {
+             unsigned int const idx = rank * fileCt + file;
+
+             bool fileNmIsRankFileIndependent;
+
+             computeInputFileName(pattern,
+                                  indexRow[file][0],
+                                  indexRow[file][1],
+                                  &inputFileList[idx],
+                                  &fileNmIsRankFileIndependent);
+
+             if (fileNmIsRankFileIndependent && !warnedSingleFile) {
+                 pm_message("Warning: No grid location (%%a/%%d) specified "
+                            "in input file pattern '%s'.  "
+                            "Input is single file.  ",
+                            pattern);
+                 warnedSingleFile = true;
+             }
+         }
+    }
+
+    pnm_freepamrow(indexRow);
+    pm_close(ifP);
+
+    *inputFileListP = inputFileList;
+    *rankCtP        = rankCt;
+    *fileCtP        = fileCt;
+}
+
+
+
 static void
 createInFileListFmPattern(const char  *  const pattern,
-                          unsigned int   const nRank,
-                          unsigned int   const nFile,
+                          unsigned int   const rankCt,
+                          unsigned int   const fileCt,
                           const char *** const inputFileListP) {
 
     const char ** inputFileList;
     unsigned int rank, file;
     bool warnedSingleFile;
 
-    MALLOCARRAY_NOFAIL(inputFileList, nRank * nFile);
+    MALLOCARRAY_NOFAIL(inputFileList, rankCt * fileCt);
 
-    for (rank = 0, warnedSingleFile = false; rank < nRank ; ++rank) {
-         for (file = 0; file < nFile ; ++file) {
-             const unsigned int idx = rank * nFile + file;
+    for (rank = 0, warnedSingleFile = false; rank < rankCt ; ++rank) {
+         for (file = 0; file < fileCt ; ++file) {
+             const unsigned int idx = rank * fileCt + file;
 
              bool fileNmIsRankFileIndependent;
 
@@ -413,14 +491,14 @@ createInFileListFmPattern(const char  *  const pattern,
 
 static void
 destroyInFileList(const char ** const inputFileList,
-                  unsigned int  const nRank,
-                  unsigned int  const nFile) {
+                  unsigned int  const rankCt,
+                  unsigned int  const fileCt) {
 
-    unsigned int const fileCt = nRank * nFile;
+    unsigned int const inFileCt = rankCt * fileCt;
 
     unsigned int fileSeq;
 
-    for (fileSeq = 0; fileSeq < fileCt; ++fileSeq)
+    for (fileSeq = 0; fileSeq < inFileCt; ++fileSeq)
         pm_strfree(inputFileList[fileSeq]);
 
     free(inputFileList);
@@ -429,30 +507,32 @@ destroyInFileList(const char ** const inputFileList,
 
 
 typedef struct {
-    unsigned int nRank;  /* Number of images in the vertical direction */
-    unsigned int nFile;  /* Number of images in the horizontal direction */
-    unsigned int hoverlap;    /* horizontal overlap */
-    unsigned int voverlap;    /* vertical overlap */
-    const char ** list;  /* List (1-dimensional array) of filenames */
-                         /* Row-major, top to bottom, left to right */
+    unsigned int  rankCt;    /* Number of images in the vertical direction */
+    unsigned int  fileCt;    /* Number of images in the horizontal direction */
+    unsigned int  hoverlap;  /* horizontal overlap */
+    unsigned int  voverlap;  /* vertical overlap */
+    const char ** list;
+        /* List (1-dimensional array) of filenames
+           Row-major, top to bottom, left to right
+        */
 } InputFiles;
 
 
 
 static const char *
-inputFileName(InputFiles     const inputFiles,
-              unsigned int   const rank,
-              unsigned int   const file) {
+inputFileName(InputFiles   const inputFiles,
+              unsigned int const rank,
+              unsigned int const file) {
 /*----------------------------------------------------------------------------
     A selected entry from "inputFiles.list" based on "rank" and "file".
 
     Currently we assume that the list is a one-dimensional represetation
     of an array, row-major, top to bottom and left to right in each row.
 ----------------------------------------------------------------------------*/
-    assert(rank < inputFiles.nRank);
-    assert(file < inputFiles.nFile);
+    assert(rank < inputFiles.rankCt);
+    assert(file < inputFiles.fileCt);
 
-    return inputFiles.list[rank * inputFiles.nFile + file];
+    return inputFiles.list[rank * inputFiles.fileCt + file];
 }
 
 
@@ -509,14 +589,14 @@ static void
 getOutputWidth(InputFiles const inputFiles,
                int *      const widthP) {
 /*----------------------------------------------------------------------------
-   Get the output width by adding up the widths of all 'inputFiles.nFile'
+   Get the output width by adding up the widths of all 'inputFiles.fileCt'
    images of the top rank, and allowing for overlap of 'inputFiles.hoverlap'
    pixels.
 -----------------------------------------------------------------------------*/
     double       totalWidth;
     unsigned int file;
 
-    for (file = 0, totalWidth = 0; file < inputFiles.nFile; ++file) {
+    for (file = 0, totalWidth = 0; file < inputFiles.fileCt; ++file) {
         struct pam inpam;
 
         getImageInfo(inputFiles, 0, file, &inpam);
@@ -528,7 +608,7 @@ getOutputWidth(InputFiles const inputFiles,
         else {
             totalWidth += inpam.width;
 
-            if (file < inputFiles.nFile-1)
+            if (file < inputFiles.fileCt-1)
                 totalWidth -= inputFiles.hoverlap;
         }
     }
@@ -541,14 +621,14 @@ static void
 getOutputHeight(InputFiles const inputFiles,
                 int *      const heightP) {
 /*----------------------------------------------------------------------------
-   Get the output height by adding up the widths of all 'inputFiles.nRank'
+   Get the output height by adding up the widths of all 'inputFiles.rankCt'
    images of the left file, and allowing for overlap of 'inputFiles.voverlap'
    pixels.
 -----------------------------------------------------------------------------*/
     double       totalHeight;
     unsigned int rank;
 
-    for (rank = 0, totalHeight = 0; rank < inputFiles.nRank; ++rank) {
+    for (rank = 0, totalHeight = 0; rank < inputFiles.rankCt; ++rank) {
         struct pam inpam;
 
         getImageInfo(inputFiles, rank, 0, &inpam);
@@ -560,7 +640,7 @@ getOutputHeight(InputFiles const inputFiles,
 
         totalHeight += inpam.height;
 
-        if (rank < inputFiles.nRank-1)
+        if (rank < inputFiles.rankCt-1)
             totalHeight -= inputFiles.voverlap;
     }
     *heightP = (int) totalHeight;
@@ -568,6 +648,42 @@ getOutputHeight(InputFiles const inputFiles,
 
 
 
+static InputFiles
+inputFileList(const char * const indexFileNm,
+              const char * const listFileNm,
+              const char * const inputFilePattern,
+              unsigned int const across,
+              unsigned int const down,
+              unsigned int const hoverlap,
+              unsigned int const voverlap) {
+
+    InputFiles inputFiles;
+
+    if (indexFileNm) {
+        createInFileListFmIdxFile(indexFileNm, inputFilePattern,
+                                  &inputFiles.rankCt,
+                                  &inputFiles.fileCt,
+                                  &inputFiles.list);
+    } else {
+        inputFiles.fileCt = across;
+        inputFiles.rankCt = down;
+
+        if (listFileNm)
+            createInFileListFmFile(listFileNm, down, across,
+                                   &inputFiles.list);
+        else
+            createInFileListFmPattern(inputFilePattern, down, across,
+                                      &inputFiles.list);
+    }
+
+    inputFiles.hoverlap = hoverlap;
+    inputFiles.voverlap = voverlap;
+
+    return inputFiles;
+}
+
+
+
 static void
 initOutpam(InputFiles   const inputFiles,
            FILE *       const ofP,
@@ -590,8 +706,8 @@ initOutpam(InputFiles   const inputFiles,
    Therefore, Caller must check all the input images to make sure they are
    consistent with the information we return.
 -----------------------------------------------------------------------------*/
-    assert(inputFiles.nFile >= 1);
-    assert(inputFiles.nRank >= 1);
+    assert(inputFiles.fileCt >= 1);
+    assert(inputFiles.rankCt >= 1);
 
     outpamP->size        = sizeof(*outpamP);
     outpamP->len         = PAM_STRUCT_SIZE(tuple_type);
@@ -614,7 +730,7 @@ initOutpam(InputFiles   const inputFiles,
 
 
 static void
-openInStreams(struct pam         inpam[],
+openInStreams(struct pam * const inpam,  /* array */
               unsigned int const rank,
               InputFiles   const inputFiles) {
 /*----------------------------------------------------------------------------
@@ -627,7 +743,7 @@ openInStreams(struct pam         inpam[],
 -----------------------------------------------------------------------------*/
     unsigned int file;
 
-    for (file = 0; file < inputFiles.nFile; ++file) {
+    for (file = 0; file < inputFiles.fileCt; ++file) {
         FILE * const ifP = pm_openr(inputFileName(inputFiles, rank, file));
 
         pnm_readpaminit(ifP, &inpam[file], PAM_STRUCT_SIZE(tuple_type));
@@ -651,49 +767,16 @@ closeInFiles(struct pam         pam[],
 
 
 static void
-assembleRow(tuple              outputRow[],
-            struct pam         inpam[],
-            unsigned int const fileCt,
-            unsigned int const hOverlap) {
-/*----------------------------------------------------------------------------
-   Assemble the row outputRow[] from the 'fileCt' input files
-   described out inpam[].
-
-   'hOverlap', which is meaningful only when 'fileCt' is greater than 1,
-   is the amount by which files overlap each other.  We assume every
-   input image is at least that wide.
-
-   We assume that outputRow[] is allocated wide enough to contain the
-   entire assembly.
------------------------------------------------------------------------------*/
-    tuple * inputRow;
-    unsigned int fileSeq;
-
-    for (fileSeq = 0, inputRow = &outputRow[0]; fileSeq < fileCt; ++fileSeq) {
-
-        unsigned int const overlap = fileSeq == fileCt - 1 ? 0 : hOverlap;
-
-        assert(hOverlap <= inpam[fileSeq].width);
-
-        pnm_readpamrow(&inpam[fileSeq], inputRow);
-
-        inputRow += inpam[fileSeq].width - overlap;
-    }
-}
-
-
-
-static void
-allocInpam(unsigned int  const rankCount,
+allocInpam(InputFiles    const inputFiles,
            struct pam ** const inpamArrayP) {
 
     struct pam * inpamArray;
 
-    MALLOCARRAY(inpamArray, rankCount);
+    MALLOCARRAY(inpamArray, inputFiles.fileCt);
 
     if (inpamArray == NULL)
         pm_error("Unable to allocate array for %u input pam structures.",
-                 rankCount);
+                 inputFiles.fileCt);
 
     *inpamArrayP = inpamArray;
 }
@@ -703,12 +786,12 @@ allocInpam(unsigned int  const rankCount,
 
 static void
 verifyRankFileAttributes(struct pam *       const inpam,
-                         unsigned int       const nFile,
+                         unsigned int       const fileCt,
                          const struct pam * const outpamP,
                          unsigned int       const hoverlap,
                          unsigned int       const rank) {
 /*----------------------------------------------------------------------------
-   Verify that the 'nFile' images that make up a rank, which are described
+   Verify that the 'fileCt' images that make up a rank, which are described
    by inpam[], are consistent with the properties of the assembled image
    *outpamP.
 
@@ -723,7 +806,7 @@ verifyRankFileAttributes(struct pam *       const inpam,
     unsigned int file;
     unsigned int totalWidth;
 
-    for (file = 0, totalWidth = 0; file < nFile; ++file) {
+    for (file = 0, totalWidth = 0; file < fileCt; ++file) {
         struct pam * const inpamP = &inpam[file];
 
         if (inpamP->depth != outpamP->depth)
@@ -750,7 +833,7 @@ verifyRankFileAttributes(struct pam *       const inpam,
         else {
             totalWidth += inpamP->width;
 
-            if (file < nFile-1)
+            if (file < fileCt-1)
                 totalWidth -= hoverlap;
         }
     }
@@ -762,10 +845,116 @@ verifyRankFileAttributes(struct pam *       const inpam,
 
 
 
+static void
+assembleRowPbm(unsigned char *    const bitrow,
+               const struct pam * const inpam,  /* array */
+               unsigned int       const fileCt,
+               unsigned int       const hOverlap) {
+/*----------------------------------------------------------------------------
+   Assemble the row outputRow[] from the 'fileCt' input files described out
+   inpam[].
+
+   'hOverlap', which is meaningful only when 'fileCt' is greater than 1, is
+   the amount by which files overlap each other.  We assume every input image
+   is at least that wide.
+
+   Use 'bitrow' as a scratch buffer to assemble the row.  It is large enough
+   to contain the entire assembly.
+-----------------------------------------------------------------------------*/
+    unsigned int fileSeq;
+    unsigned int offset;
+
+    for (fileSeq = offset = 0; fileSeq < fileCt; ++fileSeq) {
+
+        assert(fileCt > 0);
+
+        unsigned int const overlap = fileSeq == fileCt - 1 ? 0 : hOverlap;
+
+        assert(hOverlap <= inpam[fileSeq].width);
+
+        pbm_readpbmrow_bitoffset(inpam[fileSeq].file, bitrow,
+                                 inpam[fileSeq].width,
+                                 inpam[fileSeq].format, offset);
+
+        offset += inpam[fileSeq].width - overlap;
+    }
+}
+
+
+
+static void
+assembleTilesPbm(const struct pam * const outpamP,
+                 InputFiles         const inputFiles,
+                 struct pam *       const inpam,  /* array */
+                 unsigned char *    const bitrow) {
+
+    unsigned int rank;
+        /* Number of the current rank (horizontal slice).  Ranks are numbered
+           sequentially starting at 0.
+        */
+
+    unsigned int const rankCt   = inputFiles.rankCt;
+    unsigned int const fileCt   = inputFiles.fileCt;
+    unsigned int const hoverlap = inputFiles.hoverlap;
+    unsigned int const voverlap = inputFiles.voverlap;
+
+    for (rank = 0; rank < rankCt; ++rank) {
+        unsigned int row;
+        unsigned int rankHeight;
+
+        openInStreams(inpam, rank, inputFiles);
+
+        verifyRankFileAttributes(inpam, fileCt, outpamP, hoverlap, rank);
+
+        rankHeight = inpam[0].height - (rank == rankCt-1 ? 0 : voverlap);
+
+        for (row = 0; row < rankHeight; ++row) {
+            assembleRowPbm(bitrow, inpam, fileCt, hoverlap);
+            pbm_writepbmrow_packed(stdout, bitrow, outpamP->width, 0);
+        }
+        closeInFiles(inpam, fileCt);
+    }
+}
+
+
+
+static void
+assembleRow(tuple              outputRow[],
+            struct pam         inpam[],
+            unsigned int const fileCt,
+            unsigned int const hOverlap) {
+/*----------------------------------------------------------------------------
+   Assemble the row outputRow[] from the 'fileCt' input files described out
+   inpam[].
+
+   'hOverlap', which is meaningful only when 'fileCt' is greater than 1, is
+   the amount by which files overlap each other.  We assume every input image
+   is at least that wide.
+
+   We assume that outputRow[] is allocated wide enough to contain the entire
+   assembly.
+-----------------------------------------------------------------------------*/
+    tuple * inputRow;
+    unsigned int fileSeq;
+
+    for (fileSeq = 0, inputRow = &outputRow[0]; fileSeq < fileCt; ++fileSeq) {
+
+        unsigned int const overlap = fileSeq == fileCt - 1 ? 0 : hOverlap;
+
+        assert(hOverlap <= inpam[fileSeq].width);
+
+        pnm_readpamrow(&inpam[fileSeq], inputRow);
+
+        inputRow += inpam[fileSeq].width - overlap;
+    }
+}
+
+
+
 static void
 assembleTiles(struct pam * const outpamP,
               InputFiles   const inputFiles,
-              struct pam         inpam[],
+              struct pam * const inpam,  /* array */
               tuple *      const tuplerow) {
 
     unsigned int rank;
@@ -773,27 +962,27 @@ assembleTiles(struct pam * const outpamP,
            sequentially starting at 0.
         */
 
-    unsigned int const nRank    = inputFiles.nRank;
-    unsigned int const nFile    = inputFiles.nFile;
+    unsigned int const rankCt   = inputFiles.rankCt;
+    unsigned int const fileCt   = inputFiles.fileCt;
     unsigned int const hoverlap = inputFiles.hoverlap;
     unsigned int const voverlap = inputFiles.voverlap;
 
-    for (rank = 0; rank < nRank; ++rank) {
+    for (rank = 0; rank < rankCt; ++rank) {
         unsigned int row;
         unsigned int rankHeight;
 
         openInStreams(inpam, rank, inputFiles);
 
-        verifyRankFileAttributes(inpam, nFile, outpamP, hoverlap, rank);
+        verifyRankFileAttributes(inpam, fileCt, outpamP, hoverlap, rank);
 
-        rankHeight = inpam[0].height - (rank == nRank-1 ? 0 : voverlap);
+        rankHeight = inpam[0].height - (rank == rankCt-1 ? 0 : voverlap);
 
         for (row = 0; row < rankHeight; ++row) {
-            assembleRow(tuplerow, inpam, nFile, hoverlap);
+            assembleRow(tuplerow, inpam, fileCt, hoverlap);
 
             pnm_writepamrow(outpamP, tuplerow);
         }
-        closeInFiles(inpam, nFile);
+        closeInFiles(inpam, fileCt);
     }
 }
 
@@ -809,39 +998,42 @@ main(int argc, const char ** argv) {
         /* malloc'ed.  inpam[x] is the pam structure that controls the
            current rank of file x.
         */
-    tuple * tuplerow;
 
     pm_proginit(&argc, argv);
 
     parseCommandLine(argc, argv, &cmdline);
 
-    allocInpam(cmdline.across, &inpam);
+    inputFiles =
+        inputFileList(cmdline.indexfileSpec ? cmdline.indexfile : NULL,
+                      cmdline.listfileSpec ? cmdline.listfile : NULL,
+                      cmdline.inputFilePattern,
+                      cmdline.across, cmdline.down,
+                      cmdline.hoverlap, cmdline.voverlap);
 
-    if (cmdline.listfileSpec)
-        createInFileListFmFile(cmdline.listfile,
-                               cmdline.down, cmdline.across,
-                               &inputFiles.list);
-    else
-        createInFileListFmPattern(cmdline.inputFilePattern,
-                                  cmdline.down, cmdline.across,
-                                  &inputFiles.list);
-
-    inputFiles.nFile    = cmdline.across;
-    inputFiles.nRank    = cmdline.down;
-    inputFiles.hoverlap = cmdline.hoverlap;
-    inputFiles.voverlap = cmdline.voverlap;
+    allocInpam(inputFiles, &inpam);
 
     initOutpam(inputFiles, stdout, cmdline.verbose, &outpam);
 
-    tuplerow = pnm_allocpamrow(&outpam);
-
     pnm_writepaminit(&outpam);
 
-    assembleTiles(&outpam, inputFiles, inpam, tuplerow);
+    if (PNM_FORMAT_TYPE(outpam.format) == PBM_TYPE) {
+        unsigned char * const bitrow = pbm_allocrow_packed(outpam.width);
+        bitrow[pbm_packed_bytes(outpam.width) - 1] = 0x00;
 
-    pnm_freepamrow(tuplerow);
-    destroyInFileList(inputFiles.list, inputFiles.nRank, inputFiles.nFile);
+        assembleTilesPbm(&outpam, inputFiles, inpam, bitrow);
+        pnm_freerow(bitrow);
+
+      } else {
+        tuple * const tuplerow = pnm_allocpamrow(&outpam);
+        assembleTiles(&outpam, inputFiles, inpam, tuplerow);
+        pnm_freepamrow(tuplerow);
+      }
+
+    destroyInFileList(inputFiles.list, inputFiles.rankCt, inputFiles.fileCt);
     free(inpam);
 
     return 0;
 }
+
+
+
